
package cz.upol.pja.lecture07.beans;

import cz.upol.pja.lecture07.model.Quote;
import cz.upol.pja.lecture07.api.QuoteManagement;
import cz.upol.pja.lecture07.api.QuoteTest;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javax.ejb.EJB;
import javax.ejb.Stateful;
import javax.enterprise.context.SessionScoped;

@Stateful
@SessionScoped
public class QuoteTestBean implements QuoteTest, Serializable {
    
    @EJB
    private QuoteManagement quoteManagement;
    
    private int correctAnswers;
    private List<Quote> quotes;
    private List<String> answers;
    private int index = -1;
    
    @Override
    public void startNewTest(int count) {
        index = 0;
        correctAnswers = 0;
        answers = new ArrayList<>(count);
       
        List<Quote> allQuotes = quoteManagement.listAllQuotes();
        Collections.shuffle(allQuotes);
        quotes = allQuotes.subList(0, Math.min(count, allQuotes.size()));
    }

    @Override
    public String getCurrentQuery() {
        if ((quotes == null) || (index >= quotes.size())) return null;
        return quotes.get(index).getText();
    }

    @Override
    public List<String> getChoices(int count) {
        if ((quotes == null) || (index >= quotes.size())) return Collections.emptyList();
        
        Random rnd = new Random();
        
        List<String> allAuthors = new ArrayList(quoteManagement.listAuthors());
        List<String> choices = new ArrayList<>();
        
        String author = quotes.get(index).getAuthor();
        do {
            allAuthors.remove(author);
            choices.add(author);
          
            author = allAuthors.get(rnd.nextInt(allAuthors.size()));
        } while ((choices.size() < count) && (!allAuthors.isEmpty()));
        
        return Collections.unmodifiableList(choices);
    }

    @Override
    public void setAnswer(String choice) {
        answers.add(choice);
        if (choice.equals(quotes.get(index).getAuthor())) correctAnswers++;
        if (index < quotes.size()) index++;
    }
    
    @Override
    public Map<Quote, String> getAnswers() {
        if (quotes == null) return Collections.emptyMap();
        
        Map<Quote, String> answerMap = new LinkedHashMap<>();
        for (int i = 0; i < quotes.size(); i++) {
            if (i < answers.size()) 
                answerMap.put(quotes.get(i), answers.get(i));
            else
                answerMap.put(quotes.get(i), "N/A");
        }
        return Collections.unmodifiableMap(answerMap);
    }

    @Override
    public int getAnsweredQuestions() {
        return index;
    }

    @Override
    public int getTotalQuestions() {
        if (quotes == null) return -1;
        return quotes.size();
    }

    @Override
    public int getCorrectAnswers() {
        return correctAnswers;
    }
    
}
